<?php
/**
* SEF Advance extension for K2
*
* This extension will give the SEF Advance style URLs to the K2 component
* Place this file (sef_ext.php) in the main component directory: components/com_k2/
*
* Works with SEF Advance 2008 v2.3.1 or later for Joomla! 1.5 and 1.7
*
* Copyright (C) 2010 Emir Sakic, http://www.sakic.net, All rights reserved.
*
* v1.4 (2011-11-30)
**/

defined( '_JEXEC' ) or die( 'Restricted access' );

class sef_k2 {

	var $skip_prefix = false;

	/**
	* Creates the SEF Advance URL out of the request
	* Input: $string, string, The request URL (index.php?option=com_example&Itemid=$Itemid)
	* Output: $sefstring, string, SEF Advance URL ($var1/$var2/)
	**/
	function create ($string) {
		global $where, $uniqitem;
		
		if (empty($where)) {
			$where = 'alias';
		}
		
		if (empty($uniqitem) && class_exists('SefAdvanceConfig')) {
			$uniqitem = SefAdvanceConfig::get('uniqitem');
		}
		
		$db =& JFactory::getDBO();
		
		$sefstring = '';
		
		// cut the slug
		$string = preg_replace('/(\x3A|:|%3A)(.*?)(&|$)/', '$3', $string);
		
		// index.php?option=com_k2&view=itemlist&layout=category&task=category&id=1&Itemid=56
		if (stristr($string,'&amp;task=category&')) {

			$temp = explode("&amp;id=", $string);
			$temp = explode("&", $temp[1]);
			$id = $temp[0];
			$tree = $this->getCategoryPath($id);
			
			$i = 0;
			foreach ($tree as $leaf) {
				$tree[$i]['title'] = $where=='title' ? sefencode($leaf['title']) : $leaf['title'];
				$i++;
			}
			
			// check if the oldest parent is linked to menu
			$id = $tree[0]['id'];
			$query = "SELECT alias FROM #__menu WHERE link LIKE 'index.php?option=com_k2&%&task=category&id=$id' AND published=1";
			$db->setQuery( $query );
			$menu_alias = $db->loadResult();
			
			if ($menu_alias) {
				// take out the prefix k2
				$this->skip_prefix = true;
				// use menu alias instead of category title
				$tree[0]['title'] = $menu_alias;
			}

			foreach ($tree as $leaf) {
				$sefstring .= $leaf['title'] . '/';
			}

		}
		
		// index.php?option=com_k2&view=item&id=1&Itemid=56
		if (stristr($string,'&amp;view=item&')) {
			$temp = explode('&amp;id=', $string);
			$temp = explode('&', $temp[1]);
			$id = $temp[0];
			
			if (stristr($string,'&amp;task=download&')) {
				// index.php?option=com_k2&view=item&task=download&id=1&Itemid=58
				$sefstring .= 'download/'.$id.'/';
			} else if (stristr($string,'&amp;task=add')) {
				// index.php?option=com_k2&view=item&task=add
				$sefstring .= 'add/';
			} else if (stristr($string,'&amp;task=edit')) {
				// index.php?option=com_k2&view=item&task=edit
				$sefstring .= 'edit/';
				if (stristr($string,'&amp;cid=')) {
					$temp = explode('&amp;cid=', $string);
					$temp = explode('&', $temp[1]);
					$cid = $temp[0];
					$sefstring .= $cid.'/';
				}
			} else {
			
				$query = "SELECT $where AS title, catid, DATE_FORMAT(created,'%Y%m%d') AS date FROM #__k2_items WHERE id='$id'";
				$db->setQuery($query);
				$row = $db->loadObject();
				$catid = $row->catid;
				$title = $row->title;
				$title = $where=='title' ? sefencode($title) : $title;
				$date = $row->date;
				
				// get the tree
				$tree = $this->getCategoryPath($catid);
				
				$i = 0;
				foreach ($tree as $leaf) {
					$tree[$i]['title'] = $where=='title' ? sefencode($leaf['title']) : $leaf['title'];
					$i++;
				}
				
				// check if the oldest parent is linked to menu
				$id = $tree[0]['id'];
				$query = "SELECT alias FROM #__menu WHERE link LIKE 'index.php?option=com_k2&%&task=category&id=$id' AND published=1";
				$db->setQuery( $query );
				$menu_alias = $db->loadResult();
				
				if ($menu_alias) {
					// take out the prefix k2
					$this->skip_prefix = true;
					// use menu alias instead of category title
					$tree[0]['title'] = $menu_alias;
				}
				
				foreach ($tree as $leaf) {
					$sefstring .= $leaf['title'] . '/';
				}
				
				$sefstring .= $title;
				
				if ($uniqitem) {
					// YYYYMMDDID
					$sefstring .= _SEF_SPACE.$date.$id;
				}
				
				$sefstring .= '/';
			
			}
		}
		
		// index.php?option=com_k2&view=itemlist&task=user&id=62&Itemid=56
		if (stristr($string,'&amp;task=user&')) {
			$temp = explode('&amp;id=', $string);
			$temp = explode('&', $temp[1]);
			$id = $temp[0];
			
			$query = "SELECT username FROM #__users WHERE id='$id'";
			$db->setQuery($query);
			$username = $db->loadResult();
			
			// check if linked to menu
			$query = "SELECT alias FROM #__menu WHERE link LIKE 'index.php?option=com_k2&%&id=$id&task=user' AND published=1";
			$db->setQuery( $query );
			$menu_alias = $db->loadResult();
			
			if ($menu_alias) {
				// take out the prefix k2
				$this->skip_prefix = true;
				$sefstring .= $menu_alias . '/';
			} else {
				$sefstring .= 'user/' . sefencode($username) . '/';
			}
		}
		
		// index.php?option=com_k2&view=itemlist&task=tag&tag=X10&Itemid=56
		if (stristr($string,'&amp;task=tag&')) {
			$temp = explode('&amp;tag=', $string);
			$temp = explode('&', $temp[1]);
			$tag = $temp[0];
			
			// check if linked to menu
			$query = "SELECT alias FROM #__menu WHERE link LIKE 'index.php?option=com_k2&%&tag=$tag&task=tag' AND published=1";
			$db->setQuery( $query );
			$menu_alias = $db->loadResult();
			
			if ($menu_alias) {
				// take out the prefix k2
				$this->skip_prefix = true;
				$sefstring .= $menu_alias . '/';
			} else {
				$sefstring .= 'tag/' . sefencode($tag) . '/';
			}
		}
		
		// index.php?option=com_k2&view=latest&layout=latest&Itemid=61
		if (stristr($string,'&amp;view=latest&')) {
			$temp = explode('&amp;Itemid=', $string);
			$temp = explode('&', $temp[1]);
			$Itemid = $temp[0];
			
			// get it from the menu
			$query = "SELECT alias FROM #__menu WHERE id='$Itemid' AND published=1";
			$db->setQuery( $query );
			$menu_alias = $db->loadResult();
			
			if ($menu_alias) {
				// take out the prefix k2
				$this->skip_prefix = true;
				$sefstring .= $menu_alias . '/';
			}
		}
		
		
		// Tools module
		// index.php?option=com_k2&view=itemlist&task=search
		if (stristr($string,'&amp;task=search')) {
			$sefstring .= 'search/';
		}
		// index.php?option=com_k2&view=itemlist&task=date&month=6&year=2010
		if (stristr($string,'&amp;task=date')) {
			$temp = explode('task=date&amp;', $string);
			$query_str = isset($temp[1]) ? '?'.$temp[1] : '';
			$query_str = str_replace('&amp;','&',$query_str);
			$sefstring .= 'date/' . $query_str;
		}
		
		// Login module
		// index.php?option=com_k2&view=comments&tmpl=component
		if (stristr($string,'&amp;view=comments')) {
			$sefstring .= 'comments/';
		}
		
		if (stristr($string,'&amp;format=feed')) {
			$sefstring .= '?format=feed';
			if (stristr($string,'&amp;type=')) {
				$temp = explode('&amp;type=', $string);
				$temp = explode('&', $temp[1]);
				$sefstring .= '&type=' . $temp[0];
			}
		}
		
		if (stristr($string,'&amp;limitstart=')) {
			$temp = explode('&amp;limitstart=', $string);
			$temp = explode('&', $temp[1]);
			$limitstart = $temp[0];
			
			if (stristr($string,'&amp;view=item&') || stristr($string,'&amp;task=category&')) {
				// content style pagination: /$limistart/ for items and categories
				$sefstring .= !empty($limitstart) ? "$limitstart/" : '';
			} else {
				$sefstring .= !empty($limitstart) ? '?limitstart='.$limitstart : '';
			}
		}
		if (stristr($string,'&amp;showall=1')) {
			if (stristr($string,'&amp;view=item&') || stristr($string,'&amp;task=category&')) {
				// content style pagination: /all/ for items and categories
				$sefstring .= 'all/';
			} else {
				$sefstring .= '?showall=1';
			}
		}
		
		return $sefstring;
	}

	/**
	* Reverts to the query string out of the SEF Advance URL
	* Input:
	*    $url_array, array, The SEF Advance URL split in arrays
	*    $pos, int, The position offset for virtual directories (first virtual directory, which is the component name, begins at $pos+1)
	* Output: $QUERY_STRING, string, query string (var1=$var1&var2=$var2)
	*    Note that this will be added to already defined first part (option=com_example&Itemid=$Itemid)
	**/
	function revert ($url_array, $pos) {
		global $where, $uniqitem;
		
		if (empty($where)) {
			$where = 'alias';
		}
		
		if (empty($uniqitem) && class_exists('SefAdvanceConfig')) {
			$uniqitem = SefAdvanceConfig::get('uniqitem');
		}
		
		$db =& JFactory::getDBO();
		
		$where_cat = $where;
		if ($where=='title') {
			$where_cat = 'name';
		}
		
		$QUERY_STRING = '';
		
		if ($url_array[$pos+2]=='user') {
			
			// k2/user/admin/
			$username = sefdecode($url_array[$pos+3]);
			$query = "SELECT id FROM #__users WHERE username='$username'";
			$db->setQuery($query);
			$id = $db->loadResult();
			
			// TODO: find out default Itemid
			$Itemid = '';
			
			$_GET['Itemid'] = $_REQUEST['Itemid'] = $Itemid;
			$_GET['view'] = $_REQUEST['view'] = 'itemlist';
			$_GET['task'] = $_REQUEST['task'] = 'user';
			$_GET['id'] = $_REQUEST['id'] = $id;
			
			// index.php?option=com_k2&Itemid=56&view=itemlist&task=user&id=62
			$QUERY_STRING .= "&view=itemlist&task=user&id=$id";
			
		} else if ($url_array[$pos+2]=='tag') {
			
			// k2/tag/x10/
			$tag = sefdecode($url_array[$pos+3]);
			
			// TODO: find out default Itemid
			$Itemid = '';
			
			$_GET['Itemid'] = $_REQUEST['Itemid'] = $Itemid;
			$_GET['view'] = $_REQUEST['view'] = 'itemlist';
			$_GET['task'] = $_REQUEST['task'] = 'tag';
			$_GET['tag'] = $_REQUEST['tag'] = $tag;
			
			// index.php?option=com_k2&Itemid=56&view=itemlist&task=tag&tag=X10
			$QUERY_STRING .= "&view=itemlist&task=tag&tag=$tag";
			
		} else if ($url_array[$pos+2]=='download') {
		
			// k2/download/1/
			$id = $url_array[$pos+3];
			
			// TODO: find out default Itemid
			$Itemid = '';
			
			$_GET['Itemid'] = $_REQUEST['Itemid'] = $Itemid;
			$_GET['view'] = $_REQUEST['view'] = 'item';
			$_GET['task'] = $_REQUEST['task'] = 'download';
			$_GET['id'] = $_REQUEST['id'] = $id;
			
			// index.php?option=com_k2&view=item&task=download&id=1&Itemid=58
			$QUERY_STRING .= "&view=item&task=download&id=1";
			
		} else if ($url_array[$pos+2]=='search') {
		
			// k2/search/
			
			// TODO: find out default Itemid
			$Itemid = '';
			
			$_GET['Itemid'] = $_REQUEST['Itemid'] = $Itemid;
			$_GET['view'] = $_REQUEST['view'] = 'itemlist';
			$_GET['task'] = $_REQUEST['task'] = 'search';
			
			// index.php?option=com_k2&view=itemlist&task=search
			$QUERY_STRING .= "&view=itemlist&task=search";
			
		} else if ($url_array[$pos+2]=='date') {
		
			// k2/date/?month=6&year=2010
			
			// TODO: find out default Itemid
			$Itemid = '';
			
			$_GET['Itemid'] = $_REQUEST['Itemid'] = $Itemid;
			$_GET['view'] = $_REQUEST['view'] = 'itemlist';
			$_GET['task'] = $_REQUEST['task'] = 'date';
			
			// index.php?option=com_k2&view=itemlist&task=date
			$QUERY_STRING .= "&view=itemlist&task=date";
			
		} else if ($url_array[$pos+2]=='comments') {
		
			// k2/comments/?tmpl=component
			
			$_GET['view'] = $_REQUEST['view'] = 'comments';
			
			// index.php?option=com_k2&view=comments&tmpl=component
			$QUERY_STRING .= "&view=comments";
			
		} else if ($url_array[$pos+2]=='add') {
		
			// k2/add/?tmpl=component
			
			$_GET['view'] = $_REQUEST['view'] = 'item';
			$_GET['task'] = $_REQUEST['task'] = 'add';
			
			// index.php?option=com_k2&view=item&task=add&tmpl=component
			$QUERY_STRING .= "&view=item&task=add";

		} else if ($url_array[$pos+2]=='edit') {
		
			// k2/edit/$cid/?tmpl=component
			
			$_GET['view'] = $_REQUEST['view'] = 'item';
			$_GET['task'] = $_REQUEST['task'] = 'edit';
			
			// index.php?option=com_k2&view=item&task=edit&cid=1375&tmpl=component
			$QUERY_STRING .= "&view=item&task=edit";
			
			if (isset($url_array[$pos+3]) && $url_array[$pos+3]!='') {
				$cid = $_GET['cid'] = $_REQUEST['cid'] = $url_array[$pos+3];
				$QUERY_STRING .= "&cid=$cid";
			}
			
		} else {
		
			// item or category
			
			// pagination
			$last = count($url_array)-2;
			if ( $last>($pos+2) && (is_numeric($url_array[$last]) || $url_array[$last]=='all') ) {
				$limitstart = $url_array[$last];
				// take out the pagination from the $url_array
				$url_array1 = array_slice($url_array,0,$last);
				$url_array2 = array_slice($url_array,$last+1);
				$url_array = array_merge($url_array1,$url_array2);
			}
			
			// unique
			if ($uniqitem) {
				// YYYYMMDDID
				$last = count($url_array)-2;
				$temp = explode(_SEF_SPACE, $url_array[$last]);
				$dateid = $temp[count($temp)-1];
				if (strlen($dateid)>8 && is_numeric($dateid)) {
					// take out unique ID
					$url_array[$last] = str_replace(_SEF_SPACE.$dateid, '', $url_array[$last]);
				}
			}
		
			$title = $url_array[count($url_array)-2];
			
			if (isset($url_array[$pos+2]) && empty($url_array[$pos+3])) {
				// check if in menu
				$query = "SELECT id, link FROM #__menu WHERE alias='$title' AND link LIKE 'index.php?option=com_k2&%' AND published=1";
				$db->setQuery( $query );
				$menu = $db->loadObject();
				if ($menu) {
					$Itemid = $menu->id;
					$_GET['Itemid'] = $_REQUEST['Itemid'] = $Itemid;
					$link = str_replace('index.php?option=com_k2&', '', $menu->link);
					
					// register variables
					$pairs = explode('&', $link);
					foreach ($pairs as $pair) {
						$temp = explode('=', $pair);
						$_GET[$temp[0]] = $_REQUEST[$temp[0]] = $temp[1];
					}
					$QUERY_STRING .= $Itemid.'&'.$link;
					if (isset($limitstart)) {
						if ($limitstart=='all') {
							$_GET['showall'] = $_REQUEST['showall'] = 1;
							$QUERY_STRING .= "&showall=1";
						} else {
							$_GET['limitstart'] = $_REQUEST['limitstart'] = $limitstart;
							$QUERY_STRING .= "&limitstart=$limitstart";
						}
					}
					return $QUERY_STRING;
				}
			}
			
			$title = $where=='title' ? sefdecode($title) : $title;
			
			// check if item
			$query = "SELECT id, catid FROM #__k2_items WHERE $where='$title' AND published=1";
			$db->setQuery( $query );
			$row = $db->loadObject();
			if ($row) {
				$id = $row->id;
				$catid = $row->catid;
				$_GET['view'] = $_REQUEST['view'] = 'item';
				$_GET['id'] = $_REQUEST['id'] = $id;
				
				// check if item linked to menu
				$link = "index.php?option=com_k2&view=item&%&id=$id";
				$query = "SELECT id, link FROM #__menu WHERE link LIKE '$link' AND published=1";
				$db->setQuery( $query );
				$menu = $db->loadObject();
				
				if ($menu) {
					$Itemid = $menu->id;
					$_GET['Itemid'] = $_REQUEST['Itemid'] = $Itemid;
					$QUERY_STRING .= $Itemid;
				} else {
					// get the tree
					$tree = $this->getCategoryPath($catid);
					// check if the oldest parent is linked to menu
					$parent_id = $tree[0]['id'];
					$link = "index.php?option=com_k2&%&task=category&id=$parent_id";
					$query = "SELECT id, link FROM #__menu WHERE link LIKE '$link' AND published=1";
					$db->setQuery( $query );
					$menu = $db->loadObject();
					if ($menu) {
						$Itemid = $menu->id;
						$_GET['Itemid'] = $_REQUEST['Itemid'] = $Itemid;
						$QUERY_STRING .= $Itemid;
					}
				}
				
				$QUERY_STRING .= "&view=item&id=$id";
				
			} else {
				// check if category
				$query = "SELECT id FROM #__k2_categories WHERE $where_cat='$title' AND published=1";
				$db->setQuery( $query );
				$id = $db->loadResult();
				if ($id) {
					$tree = $this->getCategoryPath($id);
					// check if the oldest parent is linked to menu
					$parent_id = $tree[0]['id'];
					$link = "index.php?option=com_k2&%&task=category&id=$parent_id";
					$query = "SELECT id, link FROM #__menu WHERE link LIKE '$link' AND published=1";
					$db->setQuery( $query );
					$menu = $db->loadObject();

					$Itemid = '';
					if ($menu) {
						$Itemid = $menu->id;
						$_GET['Itemid'] = $_REQUEST['Itemid'] = $Itemid;
						$link = str_replace('index.php?option=com_k2&', '', $menu->link);
						$link = preg_replace('/&id=[0-9]+/', "&id=$id", $link);
						// register variables
						$pairs = explode('&', $link);
						foreach ($pairs as $pair) {
							$temp = explode('=', $pair);
							$_GET[$temp[0]] = $_REQUEST[$temp[0]] = $temp[1];
						}
						$QUERY_STRING .= $Itemid.'&'.$link;
					} else {
						// TODO: find out default Itemid
						$Itemid = '';
						$link = "view=itemlist&layout=category&task=category&id=$id";
						$_GET['Itemid'] = $_REQUEST['Itemid'] = $Itemid;
						$_GET['view'] = $_REQUEST['view'] = 'itemlist';
						$_GET['layout'] = $_REQUEST['layout'] = 'category';
						$_GET['task'] = $_REQUEST['task'] = 'category';
						$_GET['id'] = $_REQUEST['id'] = $id;
						$QUERY_STRING .= $Itemid.'&'.$link;
					}
				}
				
			}

			if (isset($limitstart)) {
				if ($limitstart=='all') {
					$_GET['showall'] = $_REQUEST['showall'] = 1;
					$QUERY_STRING .= "&showall=1";
				} else {
					$_GET['limitstart'] = $_REQUEST['limitstart'] = $limitstart;
					$QUERY_STRING .= "&limitstart=$limitstart";
				}
			}
		
		}

		return $QUERY_STRING;
	}
	
	/**
	* get category path
	* Input: category id
	* Output: array containing hierarchical categories
	**/
	function getCategoryPath($id) {
		global $where;
		
		if (empty($where)) {
			$where = 'alias';
		}
		
		$db =& JFactory::getDBO();
		
		$where_cat = $where;
		if ($where=='title') {
			$where_cat = 'name';
		}
		
		$tree = array();
		$has_parent = true;
		while($has_parent) {
			$query = "SELECT id, parent, $where_cat AS title FROM #__k2_categories WHERE id='$id' AND published=1";
			$db->setQuery( $query );
			$row = $db->loadObject();
			$tree[] = array('id'=>$row->id, 'title'=>$row->title);
			$has_parent = $row->parent == 0 ? false : true;
			$id = $row->parent;
		}
		$tree = array_reverse($tree);
		
		//echo '<pre>'; print_r($tree); echo '</pre>';
		
		return $tree;
		
	}

}
?>